#
# Configuration for machines that should run our music server.
#
# vim: et:ts=2:sw=2:
#
{ pkgs, config, ... }:
{
  age.secrets.bandcamp.file = ./bandcamp.cfg.age;

  home-manager.users.deprekated =
    { pkgs, ... }:
    {

      # Use mopidy as our music server.
      services.mopidy = {
        enable = true;

        extensionPackages = with pkgs; [
          mopidy-iris
          mopidy-mpd
          mopidy-tidal
          (mopidy-bandcamp.overrideAttrs (prev: {
            patches = [ ./001-fix-mopidy-bandcamp.patch ];
          }))
        ];

        settings = {

          # Tidal for streaming more-commercial music.
          tidal = {
            enabled = true;
            quality = "LOSSLESS";
          };

        };

				# Probvide our bandcamp authentication as an extra config file.
				extraConfigFiles = [
          config.age.secrets.bandcamp.path
        ];

      };

      # Translate our commands over MPRIS2.
      services.mpdris2 = {
        enable = true;
        multimediaKeys = true;
        notifications = true;

        # We're streaming, baby!
        mpd.musicDirectory = null;
      };

    };

}
